#ifndef GLOBAL_H
#define GLOBAL_H
/*
* Dateiname.... global.h
* erstellt am.. 2013-5-25
* gendert am.. 2017-02-20
* Autor........ Clemens B Fahr
* email........ clemens.fahr[at]fahrnet.de
* Kommentar.... based on RFM_wireless_small_base_01 SVN 463, remove the LED2
* .............	switch to ISR approach for RF-RX
* .............	change radio parameters to receive 24.6kbaud, reduced bandwidth to 67kHz, improve sensitivity		
*/

// global includes
//------------------------------------------------------------------
#include <avr/io.h>
#include <inttypes.h>
#include "asm.h"

// global defines
//------------------------------------------------------------------
#define ON 1
#define OFF 0
#define OUT 1
#define IN 0
#define HIGH 1
#define LOW 0
#define TRUE 1
#define FALSE 0

/* hardware pin assignment between RFM01 and MCU
* SDO ---> PB0 = 1 
* SEL <--- PB2 = 3
* IRQ ---> PB6 = 9
* SDI <--- PA7 = 11
* SCK <--- PA6 = 12 */

/* hardware pin assignment for software UART on MCU
* TX ---> PA0 = 20
*/

#define RFM_SDO_PORT PORTB
#define RFM_SDO_PIN PINB
#define RFM_SDO_POUT PORTB0
#define RFM_SDO_DDR (DDRB &= ~(0x01 << RFM_SDO_POUT))
#define RFM_SDO_HIGH (RFM_SDO_PIN & (0x01 << RFM_SDO_POUT))

#define RFM_SEL_PORT PORTB
#define RFM_SEL_POUT PORTB2
#define RFM_SEL_DDR (DDRB |= (0x01 << RFM_SEL_POUT))
#define RFM_SEL(x) do { if (x==HIGH) {RFM_SEL_PORT |= (0x01 << RFM_SEL_POUT);} else {RFM_SEL_PORT &= ~(0x01 << RFM_SEL_POUT);}} while(0)

#define RFM_IRQ_PORT PORTB
#define RFM_IRQ_PIN PINB
#define RFM_IRQ_POUT PORTB6
#define RFM_IRQ_DDR (DDRB &= ~(0x01 << RFM_IRQ_POUT))
#define RFM_IRQ_HIGH (RFM_IRQ_PIN & (0x01 << RFM_IRQ_POUT))
// configure the IRQ0 input to responde on falling edge (high --> low transition) and
// enable external interrupt 0 - will only trigger upon global I-flag being set e.g. by sei()
#define RFM_IRQ_ENABLE do { MCUCR |= (0x01 << ISC01); GIMSK |= (0x01 << INT0);} while (0)
#define RFM_IRQ_CLEAR GIFR = (0x01 << INT0)

#define RFM_SDI_PORT PORTA
#define RFM_SDI_POUT PORTA7
#define RFM_SDI_DDR (DDRA |= (0x01 << RFM_SDI_POUT))
#define RFM_SDI(x) do { if (x==HIGH) {RFM_SDI_PORT |= (0x01 << RFM_SDI_POUT);} else {RFM_SDI_PORT &= ~(0x01 << RFM_SDI_POUT);}} while(0)

#define RFM_SCK_PORT PORTA
#define RFM_SCK_POUT PORTA6
#define RFM_SCK_DDR (DDRA |= (0x01 << RFM_SCK_POUT))
#define RFM_SCK(x) do { if (x==HIGH) {RFM_SCK_PORT |= (0x01 << RFM_SCK_POUT);} else {RFM_SCK_PORT &= ~(0x01 << RFM_SCK_POUT);}} while(0)

#define LED1_PORT PORTB
#define LED1_POUT PORTB1
#define LED1_DDR (DDRB |= (0x01 << LED1_POUT))
#define LED1(x) do { if (x==ON) {LED1_PORT |= (0x01 << LED1_POUT);} else {LED1_PORT &= ~(0x01 << LED1_POUT);}} while(0)
#define LED1_MAX_BLINK 2

#define DEBUG_PORT PORTA
#define DEBUG_POUT PORTA0
#define DEBUG_DDR (DDRA |= (0x01 << DEBUG_POUT))
#define DEBUGPORTLEVEL(x) do { if (x==HIGH) {DEBUG_PORT |= (0x01 << DEBUG_POUT);} else {DEBUG_PORT &= ~(0x01 << DEBUG_POUT);}} while(0)

#define UART_TX_PORT PORTA
#define UART_TX_POUT PORTA2
#define UART_TX_DDR (DDRA |= (0x01 << UART_TX_POUT))
#define UART_TX(x) do { if (x==HIGH) {UART_TX_PORT |= (0x01 << UART_TX_POUT);} else {UART_TX_PORT &= ~(0x01 << UART_TX_POUT);}} while(0)

// ---- settings for the RFM02 -------------
//#define RFM_CMD_CONFIGSET 0x9139	// xtal osc on, 200kHz bandwidth, CLK out disabled, 868MHz band, 10pF crystal load - original/ initial set
//#define RFM_CMD_CONFIGSET 0x9069	// xtal osc off, 200kHz bandwidth, CLK out disabled, 868MHz band, 11.5pF crystal load - original/ initial set
//#define RFM_CMD_CONFIGSET 0x906B	// xtal osc off, 134kHz bandwidth, CLK out disabled, 868MHz band, 11.5pF crystal load
#define RFM_CMD_CONFIGSET 0x906D	// xtal osc off, 67kHz bandwidth, CLK out disabled, 868MHz band, 11.5pF crystal load
#define RFM_CMD_FREQSET 0xA67C		// 868.3MHz
//#define RFM_CMD_DATARATE 0xC823		// 9600baud - original/ initial set
#define RFM_CMD_DATARATE 0xC80D	// 24.6kbaud
#define RFM_CMD_AFCSET 0xC6EF		// enabled, auto keep offset, AFC range +7/-8, output enabled, strobe enabled
#define RFM_CMD_DATAFLT 0xC4AC		// auto recovery , digital LPF, DQD = 4 (default acc. to recommendation)
#define RFM_CMD_LBDCLKDIV 0xC200	// CLK 1.66MHz, Vbatt = 2.25V technically not used at all

#define RFM_CMD_WKPTIMER 0xE000
#define RFM_CMD_LOWDUTY 0xCC0E

#define RFM_CMD_FIFO_DIS 0xCE88		// VDI&synchron pattern, disabled FIFO and synchron latch, 8bits FFIT
#define RFM_CMD_FIFO_EN 0xCE89		// VDI&synchron pattern, enabled FIFO, disabled synchron latch, 8bits FFIT
#define RFM_CMD_FIFO_SYNC 0xCE8B	// VDI&synchron pattern, enabled FIFO and reset synchron latch, 8bits FFIT

//#define RFM_CMD_RX_ON 0xC0D5	        //-91dBm, -6dB LNA max, always, receiver enabled - original/ initial set
//#define RFM_CMD_RX_OFF 0xC0D4	        //-91dBm, -6dB LNA max, always, receiver disabled - original/ initial set
#define RFM_CMD_RX_ON 0xC0C3	        //-97dBm, 0dB LNA max, always, receiver enabled
#define RFM_CMD_RX_OFF 0xC0C2	        //-97dBm, 0dB LNA max, always, receiver disabled

#define RFM_CMD_READSTAT 0x0000
// ---- end of RFM01 settings --------------

#define MAX_BUFFER_SIZE 25

#define SOT 0x55  // start of (UART) telegram mark

// global variables
//------------------------------------------------------------------
volatile uint8_t RX_buffer[MAX_BUFFER_SIZE];
volatile uint8_t numberReceivedBytes;
volatile uint8_t telegramComplete;
volatile uint8_t LED1_status;
volatile uint8_t LED1_blink_count;
volatile uint8_t LED1_enabled;

#endif